import { acAppStorageModule } from 'common/js/modules';

var storageMod = acAppStorageModule;

class TapeStorageController {
	constructor(
		$q,
		$compile,
		$rootScope,
		$modal,
		$scope,
		$log,
		$state,
		$dialogs,
		cvToaster,
		cvLoc,
		cvUtil,
		storageFactory,
		storageService,
		LibraryConstants,
		tapeStorageListColumnsTemplateService,
		cvBreadcrumbsTabsFactory
	) {
		this.$q = $q;
		this.$compile = $compile;
		this.$scope = $scope;
		this.$log = $log;
		this.$state = $state;
		this.$dialogs = $dialogs;
		this.cvToaster = cvToaster;
		this.$rootScope = $rootScope;
		this.$modal = $modal;
		this.cvLoc = cvLoc;
		this.cvUtil = cvUtil;
		this.storageService = storageService;
		this.LibraryConstants = LibraryConstants;
		this.tapeStorageListColumnsTemplateService = tapeStorageListColumnsTemplateService;
		this.cvBreadcrumbsTabsFactory = cvBreadcrumbsTabsFactory;
		this.storageFactory = storageFactory;
		//this._setupBreadCrumbs();
		this.initializer();
	}

	initializer() {
		this.$scope.storageListGridOptions = {
			isKendoGrid: true
		};
		this._runGridConfig();
	}
	/** PRIVATE METHODS */
	_fetchApi() {
		const deferred = this.$q.defer();
		this.storageService
			.getAllLibraries('Tape')
			.success(data => {
				this.$scope.libraryMessage = this.cvUtil.emptyMsg();

				if (data && data instanceof Array) {
					const formattedLibs = this._setupLibraryList(data);
					deferred.resolve(formattedLibs);
				} else {
					deferred.reject({ errorMessage: this.$scope.libraryMessage });
				}
			})
			.error(e => {
				this.$scope.libraryMessage = this.cvUtil.errMsg(e);
				deferred.reject({ errorMessage: this.$scope.libraryMessage });
			});

		return deferred.promise;
	}

	_setupBreadCrumbs() {
		let breadCrumbs = [];

		breadCrumbs.push({
			title: this.cvLoc('label.nav.storage'),
			link: '#nav/mstoragePool'
		});

		this.cvBreadcrumbsTabsFactory.addBreadCrumbs(breadCrumbs);
	}

	_setupLibraryList(Libraries = []) {
		const libraries = [];
		if (Libraries.length > 0) {
			Libraries.forEach(lib => {
				const obj = {
					lib,
					link: {
						libraryId: lib.library.libraryId,
						libraryName: encodeURIComponent(lib.library.libraryName),
						model: lib.model,
						stateName: lib.stateName
					},
					permittedOptions: {
						entityId: lib.library.libraryId,
						entityName: lib.library.libraryName,
						appendToBody: true,
						permittedActionList: [
							{
								show: !this._isStandAloneLibrary(lib.libraryType),
								label: this.cvLoc('action.resetLibrary'),
								onClick: this._resetLibrary.bind(this, lib.library.libraryId)
							},
							{
								show: true,
								label: this.cvLoc('action.deconfigure'),
								onClick: this._delete.bind(this, lib.library.libraryId, lib.library.libraryName, lib.libraryType)
							},
							{
								show: !this._isStandAloneLibrary(lib.libraryType),
								label: this.cvLoc('title.exportMedia'),
								children: [
									{
										show: true,
										label: this.cvLoc('label.listBased'),
										onClick: () =>
											this.storageFactory.exportLibrary(lib.library.libraryId, lib.library.libraryName, lib.model)
									},
									{
										show: true,
										label: this.cvLoc('title.exportMediaCriteria'),
										onClick: () => this.storageFactory.exportLibraryCriteria(lib.library.libraryId)
									}
								]
							}
						]
					}
				};

				obj.permittedOptions.entity = angular.copy(obj);

				libraries.push(obj);
			});
		}

		return libraries;
	}

	_runGridConfig() {
		this.$scope.storageListGridOptions.url = options => {
			this._fetchApi()
				.then(data => {
					options.success(data);
				})
				.catch(e => options.error(e));
		};

		this.$scope.storageListGridOptions.gridToolbarMenu = [
			{
				id: 'createTape',
				disableOnDeselect: false,
				label: this.cvLoc('action.add'),
				onSelect: () => {
					this.storageFactory.addOrEditStorage(
						this.$scope,
						null, // $http - not needed
						this.storageService,
						this.$rootScope,
						this.$modal,
						null, // $stateParams - not needed
						null, // cvTableOptions - not needed
						null, // cvLoc - not needed
						'newTapeLib',
						null
					); // $state - not needed
				}
			}
		];

		this.$scope.storageListGridOptions.columns = this.tapeStorageListColumnsTemplateService.getColumns();
		this.$scope.storageListGridOptions.gridTitle = this.cvLoc('label.nav.tape');
		this.$scope.storageListGridOptions.tableName = 'TapeStorage';
		this.$scope.storageListGridOptions.idField = 'lib.library.libraryName';
		this.$scope.storageListGridOptions.hasViews = true;
		this.$scope.storageListGridOptions.pageSize = 15;
		this.$scope.storageListGridOptions.showCompanyColumnAndFilter = true;
		this.$scope.storageListGridOptions.entityInfoPath = 'lib.library.entityInfo';
		this.$scope.storageListGridOptions.sortDirection = {
			field: 'lib.library.libraryName',
			dir: 'asc'
		};

		this.$scope.storageListGridOptions.gridEmptyMessage =
			'<div class="text-align-center">' + this.cvLoc('label.noDataAvailable') + '</div>';
		this.$scope.storageListGridOptions.gridEmptySvg =
			'<svg data-cv-on-grid-empty class="add-new-placeholder" version="1.1" xmlns="http://www.w3.org/2000/svg" xmlns:xlink="http://www.w3.org/1999/xlink" x="0px" y="0px" viewBox="0 0 160 200" xml:space="preserve"><path d="M60.9,87.9h0a20.5,20.5,0,0,1-3.8-4.6V99.2h0V98.8c0,0.2,0,.5,0,0.7a13.3,13.3,0,0,0,3.9,9c6.8,7.4,21.6,12.6,38.9,12.6s32.2-5.2,38.9-12.6a13.3,13.3,0,0,0,3.9-9V83.3a20.5,20.5,0,0,1-3.8,4.6c-7.2,6.7-20.4,11.3-36.1,11.8H97c-15.7-.5-28.9-5.1-36.1-11.8h0Z"/><path d="M57.2,139.8a12.2,12.2,0,0,0,2.1,6.7c5.6,8.6,21.7,14.8,40.7,14.8s35.1-6.2,40.7-14.8a12.2,12.2,0,0,0,2.1-6.7V127.4l-0.6,1-0.3.4-0.5.6-0.3.4-0.5.6-0.3.4-0.6.6-0.3.3-0.9.9-0.3.3-0.7.6-0.4.3-0.7.5-0.5.3-0.8.5-0.4.3-1.2.8-0.3.2-1,.6-0.5.3-0.9.5-0.6.3-1,.4-0.5.2-1.3.6H127l-1.6.6-0.6.2-1.1.4-0.6.2-1.1.3-0.6.2-1.4.4h-0.4l-1.8.4h-0.5l-1.4.3h-0.7l-1.3.2h-0.7l-1.4.2h-0.6l-2,.2h-0.4l-1.7.2H92.8l-1.7-.2H90.8l-2-.2H88.2l-1.4-.2H86.1l-1.3-.2H84.2l-1.4-.3H82.3l-1.8-.4H80.1l-1.4-.4-0.6-.2-1.1-.3-0.6-.2-1.1-.4-0.6-.2-1.6-.6H72.8l-1.4-.6-0.5-.2-1-.4-0.6-.3-0.9-.5-0.5-.3-1-.6-0.3-.2-1.2-.8-0.4-.3-0.8-.5-0.5-.3-0.7-.5-0.4-.3-0.7-.6-0.3-.3-0.9-.9-0.3-.3-0.6-.6-0.3-.4-0.5-.6-0.3-.4-0.5-.6-0.3-.4-0.6-1v12.4Z"/><path d="M100,123.1c-18,0-33.2-5.3-40.4-13.1a19.2,19.2,0,0,1-2.5-3.3v12.9c0,0.3,0,.5,0,0.8,0,11.9,19.2,21.5,42.8,21.5s42.8-9.6,42.8-21.5V106.7a19.2,19.2,0,0,1-2.5,3.3C133.2,117.8,118,123.1,100,123.1Z"/><path d="M57.2,76.2c0,3.8,1.9,7.3,5.3,10.4,7.3,6.6,21.4,11.2,37.5,11.2s30.2-4.5,37.5-11.2c3.4-3.1,5.3-6.6,5.3-10.4a12.2,12.2,0,0,0-2.1-6.6C135.2,60.9,119,54.6,100,54.6S64.8,60.9,59.3,69.5A12.2,12.2,0,0,0,57.2,76.2Z"/><path d="M19.2,55.3V71.2h0V70.8c0,0.2,0,.5,0,0.7a13.3,13.3,0,0,0,3.9,9c5.9,6.4,17.8,11.1,32.1,12.3V75.9h0a13.4,13.4,0,0,1,.9-4.4c-14.4-1-26.4-5.4-33.1-11.6A20.5,20.5,0,0,1,19.2,55.3Z"/><path d="M19.2,99.4v12.4a12.2,12.2,0,0,0,2.1,6.7c4.9,7.5,17.9,13.2,33.9,14.5v-13h0v-4.4C37.9,114.3,24.2,107.9,19.2,99.4Z"/><path d="M21.6,81.9a19.2,19.2,0,0,1-2.5-3.3V91.5c0,0.3,0,.5,0,0.8,0,10.7,15.6,19.6,36,21.2V99.3h0V94.8C40.2,93.7,27.9,88.8,21.6,81.9Z"/><path d="M102.7,41.5C97.2,32.9,81,26.6,62,26.6S26.8,32.9,21.3,41.5a12.2,12.2,0,0,0-2.1,6.6c0,3.8,1.9,7.3,5.3,10.4,6.5,5.9,18.5,10.2,32.5,11l0.6-1.1C63.7,59,80.7,52.6,100,52.6h3.8a11.5,11.5,0,0,0,1-4.6A12.2,12.2,0,0,0,102.7,41.5Z"/></svg>';

		this.$scope.storageListGridOptions.beforeGridInitialize = ({ grid }) => (this.grid = grid);
		this.$scope.storageListGridOptions.onGridDataBound = this._onGridDataBound.bind(this);
	}

	_onGridDataBound(dataItem, row) {
		const permittedOptions = dataItem.permittedOptions;
		const id = permittedOptions.entityId;
		const template = `<cv-permitted-actions cv-permitted-options="permittedOptions${id}"></cv-permitted-actions>`;
		this.$scope[`permittedOptions${id}`] = permittedOptions;
		row.find('.permittedActions').append(this.$compile(template)(this.$scope));
	}

	_resetLibrary(libraryId) {
		var callBackFunctions = {
			noFunction: () => {},
			yesFunction: () => {
				this.storageService
					.resetLibrary(libraryId)
					.success(function(data) {
						if ((data.errorMessage != '') & (data.errorCode != 0)) {
							this.cvToaster.showErrorMessage({
								ttl: '10000',
								message: data.errorMessage
							});
						} else {
							this.cvToaster.showSuccessMessage({
								ttl: '10000',
								message: cvLoc('popup.succesResetLibrary')
							});
						}
						this.$log.debug('Reset Library success');
						this.$state.forceReload();
					})
					.error(error => {
						this.cvToaster.showErrorMessage({
							ttl: '10000',
							message: error ? error : cvLoc('popup.failedResetLibrary')
						});
						this.$log.error('Reset Library failed');
					});
			}
		};

		this.$dialogs.confirm(
			this.cvLoc('title.confirmResetLibrary'),
			this.cvLoc('confirm.resetLibrary'),
			callBackFunctions
		);
	}

	_delete(libraryId, libraryName, libraryType) {
		var LibraryInfo = {
			libraryEntity: {
				libraryId: libraryId,
				libraryName: libraryName,
				libraryType: libraryType
			}
		};
		var callBackFunctions = {
			noFunction: () => {},
			yesFunction: () => {
				this.storageService
					.delLibrary(LibraryInfo)
					.success(() => {
						this.$log.debug('Delete Library success');
						this.$state.forceReload();
					})
					.error(error => {
						this.$dialogs.error(cvLoc('label.error'), cvUtil.errMsg(error).message);
						this.$log.error('Delete Library failed');
					});
			}
		};
		this.$dialogs.confirm(
			this.cvLoc('label.confirmDelete'),
			this.cvLoc('label.confirmDeleteTapeLibrary') +
				'<br/>' +
				this.cvLoc('label.generalDeleteLibarry', '<b>' + libraryName + '</b>'),
			callBackFunctions
		);
	}

	/* Library type */
	_isStandAloneLibrary(libraryType) {
		return (
			libraryType == this.LibraryConstants.CV_LIB_STANDALONE_TAPE ||
			libraryType == this.LibraryConstants.CV_LIB_STANDALONE_OPTICAL
		);
	}

	/** PRIVATE METHODS */
}

TapeStorageController.$inject = [
	'$q',
	'$compile',
	'$rootScope',
	'$uibModal',
	'$scope',
	'$log',
	'$state',
	'$dialogs',
	'cvToaster',
	'cvLoc',
	'cvUtil',
	'storageFactory',
	'storageService',
	'LibraryConstants',
	'tapeStorageListColumnsTemplateService',
	'cvBreadcrumbsTabsFactory'
];

storageMod.controller('tapeStorageController', TapeStorageController);

export default storageMod;
